/*

	AeroGlassHelper for CodeGear C++Builder 2007

	Author: vcore
	
	Email: waterpure1985@gmail.com	
	
	Personal blog: http://vcore1985.pixnet.net/blog
	
	GNU GPL Licesne	

	Copyright(c) 2010

*/
#ifndef AEROGLASS_HELPER_INCLUDED
#define AEROGLASS_HELPER_INCLUDED
#include <gdiplus.h>
#include <uxtheme.h>
#include <Forms.hpp>
#include <Graphics.hpp>
#include <dwmapi.h>
class AeroGlassHelper
{
	private:
		ULONG_PTR gdiplusToken;
		Gdiplus::GdiplusStartupInput gdiplusStartupInput;
		HTHEME m_hTheme;
		void my_byte_swap ( char &a , char &b ) {
			char tmp = a;
			a = b;
			b = tmp;
		}
		bool SupportAeroGlass() {
			OSVERSIONINFO OSversion;
			OSversion.dwOSVersionInfoSize = sizeof ( OSVERSIONINFO );
			::GetVersionEx ( &OSversion );
			if ( OSversion.dwMajorVersion >= 6 ) return true;
			return false;
		}

	public:
		bool IsCompositionEnabled() const {
			HRESULT hr;
			BOOL bEnabled;
			hr = DwmIsCompositionEnabled ( &bEnabled );
			return SUCCEEDED ( hr ) && bEnabled;
		}
		void Init ( TForm *_tform ) {
			GdiplusStartup ( &gdiplusToken, &gdiplusStartupInput, NULL );
			m_hTheme = OpenThemeData ( _tform->Handle, L"WINDOW" );
		}
		void Release() {
			CloseThemeData ( m_hTheme );
		}

		void PrintText ( TForm *_tform , int x , int y , AnsiString str , AnsiString FontName , int FontSize ) {
			if ( m_hTheme == NULL ) return;
			::RECT txt_rect;
			if ( !SupportAeroGlass() || !IsCompositionEnabled() ) {

				AnsiString old_form_name = _tform->Canvas->Font->Name;
				int old_form_size = _tform->Canvas->Font->Size;
				_tform->Canvas->Font->Name = FontName;
				_tform->Canvas->Font->Size = FontSize;
				_tform->Canvas->Brush->Style = bsClear;
				_tform->Canvas->TextOut( x, y , str );
				
			} else {
				Graphics::TBitmap *bmp = new Graphics::TBitmap;
				bmp->Canvas->Font->Name = FontName;
				bmp->Canvas->Font->Size = FontSize;
				bmp->Width = bmp->Canvas->TextWidth ( str );
				bmp->Height = bmp->Canvas->TextHeight ( str );
				bmp->PixelFormat = pf32bit;
				bmp->Canvas->Brush->Style = bsSolid;
				bmp->Canvas->Brush->Color = 0;
				bmp->Canvas->FillRect ( TRect ( 0, 0, bmp->Width, bmp->Height ) );
				bmp->Canvas->Brush->Style = bsClear;
				DTTOPTS dto = { sizeof ( DTTOPTS ) };
				const UINT uFormat = DT_SINGLELINE | DT_CENTER | DT_VCENTER | DT_NOPREFIX;
				dto.dwFlags = DTT_COMPOSITED | DTT_GLOWSIZE;
				dto.iGlowSize = 5;

				txt_rect.left = 0;
				txt_rect.top = 0;
				txt_rect.right =  bmp->Width;
				txt_rect.bottom = bmp->Height;
				DrawThemeTextEx ( m_hTheme, bmp->Canvas->Handle , 0 , 0 , WideString ( str ).c_bstr() , -1,
				                  uFormat, &txt_rect , &dto );
				// Fast Bitmap Flip (only for 32bit pixel mode)
				for ( int row = 0 ; row < bmp->Height / 2 ; row++ ) {
					char *lineptr = ( char * ) bmp->ScanLine[row];
					char *lineptr2 = ( char * ) bmp->ScanLine[bmp->Height-row-1];
					for ( int col = 0 ; col < bmp->Width ; col++ ) {
						my_byte_swap ( lineptr[col*4] , lineptr2[col*4] );
						my_byte_swap ( lineptr[col*4+1] , lineptr2[col*4+1] );
						my_byte_swap ( lineptr[col*4+2] , lineptr2[col*4+2] );
						my_byte_swap ( lineptr[col*4+3] , lineptr2[col*4+3] );
					}
				}
				::BitBlt  ( _tform->Canvas->Handle , x, y , txt_rect.right - txt_rect.left, txt_rect.bottom - txt_rect.top ,
				            bmp->Canvas->Handle , 0, 0, SRCCOPY );
			}

		}
		void DrawImageFromFile ( TForm *_tform , int x , int y , AnsiString filename ) {
			Gdiplus::Graphics gr ( _tform->Canvas->Handle );
			Gdiplus::Image img ( WideString ( filename ).c_bstr() );
			gr.DrawImage ( &img, x, y );
		}
};
#endif







